'use strict';

var config = require('../../config.json');

/**
 * @param {Object} state
 * @returns {String|Boolean}
 */
function getDirective(state) {
    var directive = void 0;

    if ('useDirective' in state.opts && state.opts.useDirective !== true) {
        directive = state.opts.useDirective;
    } else {
        directive = config.default.useDirective;
    }

    return directive;
}

/**
 * @param {Object} path
 * @param {Object} state
 * @returns {String}
 */
module.exports = function findRelativeComment(path, state) {
    if (!path.node.leadingComments) {
        return null;
    }

    var comments = path.node.leadingComments;
    var directive = getDirective(state);

    var previousNode = path.key !== 0 ? path.getSibling(path.key - 1) : null;
    var previousNodeEnd = previousNode && previousNode.node ? previousNode.node.end || -1 : -1;

    var functionDeclarationStart = void 0;

    /**
     * Fix for annoying edge case, when ObjectProperty node doesn't have start index
     * Workaround: find function body and get it's start index.
     */
    if (path.isObjectProperty()) {
        functionDeclarationStart = path.node.start || path.get('value').get('body').node.start;
    } else {
        functionDeclarationStart = path.node.start;
    }

    var foundedCommentsCollection = comments.filter(function (comment) {
        return comment.start > previousNodeEnd && comment.end < functionDeclarationStart;
    });

    var foundedComment = void 0;

    if (typeof directive === 'string') {
        foundedComment = foundedCommentsCollection.find(function (comment) {
            return comment.value.includes('@' + directive);
        });
    } else {
        foundedComment = foundedCommentsCollection[foundedCommentsCollection.length - 1];
    }

    return foundedComment ? foundedComment.value : null;
};